#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "leptonica/allheaders.h"

#define MAX_FILE_LEN   512

/* Used to scale and binarize a bitmap. */
int main(int argc, char *argv[])
{
  char source_file[MAX_FILE_LEN] = "";
  char dest_file[MAX_FILE_LEN] = "";
  float scale_factor = 3.2f;
  float binarize_factor = 0.15f;
  PIX *pixs;
  PIX *pix_grayscale;
  PIX *pix_scaled;
  PIX *pix_binarize;

  /* Check for valid number of inputs */
  if(argc < 5)
  {
    printf("leptonica_util v1.0 by Chris Brochtrup\n");
    printf("Usage:   leptonica_util.exe <source bmp> <destination bmp> <scale factor (float)> <binarize factor (float, 0 = disable)> \n\n");
    printf("Example: leptonica_util.exe src.bmp dest.bmp 3.2 0.15\n\n");
    exit(1);
  }

  /* Get args */
  strcpy_s(source_file, MAX_FILE_LEN, argv[1]);
  strcpy_s(dest_file, MAX_FILE_LEN, argv[2]);
  scale_factor = atof(argv[3]);
  binarize_factor = atof(argv[4]);

  /* Read in source image and convert to grayscale */
  pixs = pixRead(source_file);
  pix_grayscale = pixConvertRGBToGray(pixs, 0.0f, 0.0f, 0.0f);
  pixDestroy(&pixs);
  
  /* Scale the image */
  pix_scaled = pixScaleGrayLI(pix_grayscale, scale_factor, scale_factor);
  pixDestroy(&pix_grayscale);
  // pixWrite("after_scale.bmp", pix_scaled, IFF_BMP); // For debug

  /* Binarize? */
  if(binarize_factor > 0)
  {
    /*  From the documentation:
        Sauvola binarization computes a local threshold based on the local average and square
        average. It takes two constants: the window size for the measurment at each pixel and a
        parameter that determines the amount of normalized local standard deviation to subtract
        from the local average value. */
    pixSauvolaBinarizeTiled(pix_scaled, 7, binarize_factor, 2, 2, NULL, &pix_binarize);

#if 0
    /* Note: When Otsu is used with Tesseract, this method works just as well as Sauvola
             (and in some cases even better).
    
             However, NHocr does not like this method at all, mostly producing garbage.
             This is the primary reason that Sauvola was chosen instead of Otsu for
             use in Capture2Text.
    
       From the documentation:
       Computes a global threshold over each tile and performs the threshold operation,
       resulting in a binary image for each tile. These are stitched into the final result. */
    pixOtsuAdaptiveThreshold(pix_scaled, 16, 16, 3, 3, 0.6, NULL, &pix_binarize); 	
#endif /* 0 */

    pixWrite(dest_file, pix_binarize, IFF_BMP);
    pixDestroy(&pix_binarize);
  }
  else
  {
    pixWrite(dest_file, pix_scaled, IFF_BMP);
  }
  
  pixDestroy(&pix_scaled);

  return 0;

} /* main */